/* eslint-disable no-magic-numbers */
import { Style } from 'ol/style';
import Feature, { FeatureLike } from 'ol/Feature';
import { Geometry, MultiPolygon } from 'ol/geom';
import { UsePointToProjectionResult } from '@/utils/map/usePointToProjection';
import getStroke from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/getStroke';
import getFill from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/getFill';
import RenderFeature from 'ol/render/Feature';
import Polygon from 'ol/geom/Polygon';
import getMultiPolygon from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/getMultiPolygon';
import getText from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/getText';
import { rgbToHex } from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/rgbToHex';
import { Shape } from '@/types/models';
import { MapInstance } from '@/types/map';
import {
  ColorObject,
  HorizontalAlign,
  VerticalAlign,
} from '@/components/Map/MapViewer/MapViewerVector/MapViewerVector.types';
import {
  BLACK_COLOR,
  WHITE_COLOR,
} from '@/components/Map/MapViewer/MapViewerVector/MapViewerVector.constants';

export type CustomMultiPolygonProps = {
  shapes: Array<Shape>;
  x: number;
  y: number;
  width: number;
  height: number;
  zIndex: number;
  fillColor?: ColorObject;
  borderColor?: ColorObject;
  fontColor?: ColorObject;
  lineWidth?: number;
  text?: string;
  fontSize?: string | number;
  nameX: number;
  nameY: number;
  nameHeight: number;
  nameWidth: number;
  nameVerticalAlign?: VerticalAlign;
  nameHorizontalAlign?: HorizontalAlign;
  pointToProjection: UsePointToProjectionResult;
  mapInstance: MapInstance;
};

export default class CustomMultiPolygon {
  multiPolygonStyle: Style;

  textStyle: Style | undefined;

  polygons: Array<Polygon> = [];

  multiPolygonFeature: Feature;

  constructor({
    shapes,
    x,
    y,
    width,
    height,
    zIndex,
    fillColor = WHITE_COLOR,
    borderColor = BLACK_COLOR,
    fontColor = BLACK_COLOR,
    lineWidth = 1,
    text = '',
    fontSize = 12,
    nameX,
    nameY,
    nameHeight,
    nameWidth,
    nameVerticalAlign = 'MIDDLE',
    nameHorizontalAlign = 'CENTER',
    pointToProjection,
    mapInstance,
  }: CustomMultiPolygonProps) {
    if (text) {
      const { textCoords, textStyle } = getText({
        text,
        fontSize,
        x: nameX,
        y: nameY,
        width: nameWidth,
        height: nameHeight,
        color: rgbToHex(fontColor),
        verticalAlign: nameVerticalAlign,
        horizontalAlign: nameHorizontalAlign,
        pointToProjection,
      });
      this.textStyle = new Style({
        geometry: (feature: FeatureLike): Geometry | RenderFeature | undefined => {
          const geometry = feature.getGeometry();
          if (geometry && geometry.getType() === 'MultiPolygon') {
            return (geometry as MultiPolygon).getPolygon(0).getInteriorPoint();
          }
          return undefined;
        },
        text: textStyle.getText() || undefined,
        zIndex,
      });
      this.polygons.push(new Polygon([[textCoords, textCoords]]));
    }

    this.polygons = [
      ...this.polygons,
      ...getMultiPolygon({ x, y, width, height, shapes, pointToProjection }),
    ];

    this.multiPolygonFeature = new Feature({
      geometry: new MultiPolygon([...this.polygons]),
      getTextScale: (resolution: number): number => {
        const maxZoom = mapInstance?.getView().getMaxZoom();
        if (maxZoom) {
          const minResolution = mapInstance?.getView().getResolutionForZoom(maxZoom);
          if (minResolution) {
            return Math.round((minResolution / resolution) * 100) / 100;
          }
        }
        return 1;
      },
    });

    this.multiPolygonStyle = new Style({
      stroke: getStroke({ color: rgbToHex(borderColor), width: lineWidth }),
      fill: getFill({ color: rgbToHex(fillColor) }),
      zIndex,
    });

    this.multiPolygonFeature.setStyle(this.styleFunction.bind(this));
  }

  styleFunction(feature: FeatureLike, resolution: number): Style | Array<Style> | void {
    const getTextScale = feature.get('getTextScale');
    let textScale = 1;
    if (getTextScale instanceof Function) {
      textScale = getTextScale(resolution);
    }
    const styles = [];
    if (this.textStyle) {
      const text = this.textStyle.getText();
      if (text) {
        this.textStyle.getText()?.setScale(textScale);
        styles.push(this.textStyle);
      }
    }
    return [...styles, this.multiPolygonStyle];
  }
}
